package {{ cookiecutter.__mvn_package }}.module;

import cn.dev33.satoken.stp.StpUtil;
import cn.dev33.satoken.temp.SaTempUtil;
import {{ cookiecutter.__mvn_package }}.common.Constants;
import {{ cookiecutter.__mvn_package }}.rest.api.R;
import {{ cookiecutter.__mvn_package }}.service.CaptchaService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.tags.Tag;
import jakarta.annotation.Resource;
import jakarta.servlet.ServletOutputStream;
import jakarta.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.IOException;
import java.net.URLEncoder;
import java.time.OffsetDateTime;
import java.util.HashMap;
import java.util.Map;
import lombok.extern.slf4j.Slf4j;
import org.dromara.hutool.core.array.ArrayUtil;
import org.dromara.hutool.core.compress.ZipUtil;
import org.dromara.hutool.core.data.id.IdUtil;
import org.dromara.hutool.core.date.DatePattern;
import org.dromara.hutool.core.date.DateTime;
import org.dromara.hutool.core.date.format.FastDateFormat;
import org.dromara.hutool.core.io.IoUtil;
import org.dromara.hutool.core.io.file.FileNameUtil;
import org.dromara.hutool.core.io.file.FileReader;
import org.dromara.hutool.core.io.file.FileUtil;
import org.dromara.hutool.core.lang.Validator;
import org.dromara.hutool.core.map.MapUtil;
import org.dromara.hutool.core.text.StrUtil;
import org.dromara.hutool.core.text.split.SplitUtil;
import org.dromara.hutool.core.util.CharsetUtil;
import org.dromara.hutool.extra.management.ManagementUtil;
import org.dromara.hutool.extra.pinyin.PinyinUtil;
import org.dromara.hutool.http.meta.ContentType;
import org.dromara.hutool.http.meta.HeaderName;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

/**
 * (PubController) 公共接口.
 *
 * @author zweiandlen
 * @since 2024/7/3 下午2:43
 */
@Slf4j
@Tag(name = "公共接口", description = "常用工具，无需令牌，部分接口需临时签名[sign]")
@RestController
@RequestMapping("pub")
public class PubController {

  @Resource CaptchaService captchaService;

  @Value("${file.path:}")
  private String path;

  @Operation(summary = "服务器时间")
  @GetMapping("/now")
  public OffsetDateTime now() {
    return OffsetDateTime.now();
  }

  /**
   * 图形验证码.
   *
   * @return 图形与ID
   */
  @Operation(summary = "图形验证码")
  @GetMapping("/captcha/img")
  public R getCaptchaImg() {
    String objectId = IdUtil.objectId();
    String captcha = captchaService.getCodeById(objectId);

    Map<String, String> result =
        MapUtil.builder(new HashMap<String, String>())
            .put("objectId", objectId)
            .put("captcha", captcha)
            .build();
    return R.ok(result);
  }

  /**
   * 邮件验证码.
   *
   * @return 邮件ID
   */
  @Operation(summary = "邮件验证码")
  @GetMapping("/captcha/email")
  public R getCaptchaEmail(String email) {
    if (!Validator.isEmail(email)) {
      throw new RuntimeException("邮箱地址格式错误");
    }
    String objectId = IdUtil.objectId();
    objectId = captchaService.getCodeByEmail(objectId, email);

    Map<String, String> result =
        MapUtil.builder(new HashMap<String, String>()).put("objectId", objectId).build();
    return R.ok(result);
  }

  /**
   * 获得拼音首字母.
   *
   * @param text 文字
   * @return 拼音首字母
   */
  @Operation(summary = "获得拼音首字母")
  @GetMapping("/pinyin/first")
  public R getPinyinFirst(String text) {
    if (StrUtil.isNotBlank(text)) {
      return R.ok(PinyinUtil.getFirstLetter(text, StrUtil.EMPTY));
    } else {
      return R.failed("不能为空");
    }
  }

  /**
   * 获得完整拼音.
   *
   * @param text 文字
   * @return 拼音
   */
  @Operation(summary = "获得完整拼音")
  @GetMapping("/pinyin/full")
  public R getPinyinFull(String text) {
    if (StrUtil.isNotBlank(text)) {
      return R.ok(PinyinUtil.getPinyin(text));
    } else {
      return R.failed("不能为空");
    }
  }

  /**
   * 获得临时签名，5分钟有效.
   *
   * @param desc 注释
   * @return 签名
   */
  @Operation(summary = "获得临时签名，5分钟有效")
  @PostMapping("/sign")
  public R getTempToken(@RequestParam(defaultValue = "缺省", required = false) String desc) {
    return R.ok(
        SaTempUtil.createToken(
            StrUtil.format("临时签名 / {} / {}", IdUtil.objectId(), desc), Constants.SIGN_EXPIRE));
  }

  /**
   * 上传文件，已登录无需签名，未登录需要签名.
   *
   * @param files 多文件
   * @param sign 签名
   * @return 存储路径
   */
  @Operation(summary = "上传文件，已登录无需签名，未登录需要签名")
  @PostMapping(value = "/file/upload", consumes = "multipart/form-data")
  public R upload(
      @RequestPart("file") MultipartFile[] files,
      @RequestParam(value = "sign", required = false) String sign) {
    // 解析签名
    if (StpUtil.isLogin()) {
      String loginId = StpUtil.getLoginIdAsString();
      log.info("登录凭据 {} 开始上传 {} 个文件", loginId, files.length);
    } else {
      String token = SaTempUtil.parseToken(sign, String.class);
      if (StrUtil.isNotBlank(token)) {
        log.info("临时签名 {} 开始上传 {} 个文件", token, files.length);
      } else {
        throw new RuntimeException("缺少签名参数");
      }
    }
    // 保存文件
    if (StrUtil.isBlank(this.path)) {
      // 默认在jar包所在位置创建 files 文件夹
      String currentDir = ManagementUtil.getUserInfo().getCurrentDir();
      File dir = FileUtil.file(currentDir, "files");
      this.path = dir.getAbsolutePath();
    }
    String[] filepath = new String[files.length];
    for (int i = 0; i < files.length; i++) {
      MultipartFile file = files[i];
      String fullname = IdUtil.objectId();
      String extName = FileNameUtil.extName(file.getOriginalFilename());
      // 文件后缀黑名单
      String[] blackList = {"jsp", "php", "exe", "dll", "vxd", "html", "htm", "js", "asp", "aspx"};
      if (ArrayUtil.containsIgnoreCase(blackList, extName)) {
        throw new RuntimeException("该文件不允许上传");
      }
      if (StrUtil.isNotBlank(extName)) {
        fullname = StrUtil.join(StrUtil.DOT, fullname, extName);
      }
      String date =
          FastDateFormat.getInstance(DatePattern.PURE_DATE_PATTERN).format(DateTime.now());
      File dest = FileUtil.file(this.path, date, fullname);
      FileUtil.mkdir(dest);
      try {
        file.transferTo(dest);
        filepath[i] = FileUtil.subPath(this.path, dest);
      } catch (IOException e) {
        throw new RuntimeException("文件保存失败！");
      }
    }
    String join = ArrayUtil.join(filepath, StrUtil.COMMA);
    log.info("文件上传成功 {}", join);
    return R.ok(join);
  }

  /**
   * 打包下载 Zip 存档，已登录无需签名，未登录需要签名.
   *
   * @param urls 多存储路径
   * @param sign 签名
   * @throws IOException IO异常
   */
  @Operation(summary = "打包下载 Zip 存档，已登录无需签名，未登录需要签名")
  @GetMapping("/file/zip")
  public void archive(
      HttpServletResponse response,
      String urls,
      @RequestParam(value = "sign", required = false) String sign)
      throws IOException {
    if (StrUtil.isBlank(urls)) {
      return;
    }
    // 解析签名
    if (StpUtil.isLogin()) {
      String loginId = StpUtil.getLoginIdAsString();
      log.info("登录凭据 {} 开始打包文件 {}", loginId, urls);
    } else {
      String token = SaTempUtil.parseToken(sign, String.class);
      if (StrUtil.isNotBlank(token)) {
        log.info("临时签名 {} 开始打包文件 {}", token, urls);
      } else {
        throw new RuntimeException("缺少签名参数");
      }
    }
    String path;
    if (StrUtil.isBlank(this.path)) {
      String currentDir = ManagementUtil.getUserInfo().getCurrentDir();
      File files = FileUtil.file(currentDir, "files");
      path = StrUtil.replace(files.getAbsolutePath(), StrUtil.BACKSLASH, StrUtil.SLASH);
      path = StrUtil.appendIfMissing(path, StrUtil.SLASH);
    } else {
      path = this.path;
    }
    String finalPath = path;
    String[] fileStr =
        SplitUtil.split(urls, StrUtil.COMMA).stream()
            .map(item -> (finalPath + item))
            .toArray(String[]::new);
    File[] files = new File[fileStr.length];
    for (int i = 0; i < fileStr.length; i++) {
      files[i] = FileUtil.file(fileStr[i]);
    }
    response.reset();
    response.setHeader(HeaderName.CONTENT_TYPE.getValue(), ContentType.OCTET_STREAM.getValue());
    response.setHeader(
        HeaderName.CONTENT_DISPOSITION.getValue(),
        "attachment; filename=" + URLEncoder.encode(IdUtil.objectId() + ".zip", CharsetUtil.UTF_8));
    ServletOutputStream out = response.getOutputStream();
    FileReader.of(ZipUtil.zip(FileUtil.createTempFile(), false, files)).writeToStream(out);
    IoUtil.closeQuietly(out);
    log.info("文件打包成功，共 {} 个文件", files.length);
  }
}
