package {{ cookiecutter.__mvn_package }}.web;

import cn.dev33.satoken.exception.NotLoginException;
import cn.dev33.satoken.exception.NotPermissionException;
import {{ cookiecutter.__mvn_package }}.rest.api.R;
import java.util.List;
import lombok.extern.slf4j.Slf4j;
import org.dromara.hutool.core.text.StrUtil;
import org.springframework.context.support.DefaultMessageSourceResolvable;
import org.springframework.dao.DuplicateKeyException;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.MethodArgumentNotValidException;
import org.springframework.web.bind.annotation.ExceptionHandler;
import org.springframework.web.bind.annotation.RestControllerAdvice;
import org.springframework.web.servlet.resource.NoResourceFoundException;

/**
 * (GlobalControllerAdvice) 全局异常处理.
 *
 * @author zweiandlen
 * @since 2024/7/3 下午2:40
 */
@Slf4j
@RestControllerAdvice
public class GlobalControllerAdvice {

  /**
   * 参数校验错误.
   *
   * @param e 异常
   * @return 响应结果
   */
  @ExceptionHandler(MethodArgumentNotValidException.class)
  public ResponseEntity<?> argsNotValidException(MethodArgumentNotValidException e) {
    log.error("入参校验", e);
    List<String> errors =
        e.getBindingResult().getFieldErrors().stream()
            .map(DefaultMessageSourceResolvable::getDefaultMessage)
            .toList();
    String error = StrUtil.join(StrUtil.COMMA, errors);
    return ResponseEntity.badRequest().body(R.failed(error));
  }

  /**
   * 没有登录错误.
   *
   * @param e 异常
   * @return 响应结果
   */
  @ExceptionHandler(NotLoginException.class)
  public ResponseEntity<?> argsNotValidException(Exception e) {
    log.error("没有登录", e);
    return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body(R.failed(e.getMessage()));
  }

  /**
   * 权限校验错误.
   *
   * @param e 异常
   * @return 响应结果
   */
  @ExceptionHandler({NotPermissionException.class})
  public ResponseEntity<?> handleException(NotPermissionException e) {
    log.error("权限不足", e);
    return ResponseEntity.status(HttpStatus.FORBIDDEN).body(R.failed(e.getMessage()));
  }

  /**
   * 唯一约束错误.
   *
   * @param e 异常
   * @return 响应结果
   */
  @ExceptionHandler({DuplicateKeyException.class})
  public ResponseEntity<?> handleException(DuplicateKeyException e) {
    log.error("唯一约束", e);
    return ResponseEntity.badRequest().body(R.failed("违反唯一约束"));
  }

  /**
   * 静态资源错误.
   *
   * @param e 异常
   * @return 响应结果
   */
  @ExceptionHandler({NoResourceFoundException.class})
  public ResponseEntity<?> handleException(NoResourceFoundException e) {
    log.error("页面未找到 {}", e.getMessage());
    return ResponseEntity.status(HttpStatus.NOT_FOUND).body(e.getMessage());
  }

  /**
   * 其他错误.
   *
   * @param e 异常
   * @return 响应结果
   */
  @ExceptionHandler(Exception.class)
  public ResponseEntity<?> handleException(Exception e) {
    log.error("页面异常", e);
    return ResponseEntity.badRequest().body(R.failed(e.getMessage()));
  }
}
